
#ifdef BURUNDUK
#  include "my_testlib.h"
#else
#  include "testlib.h"
#endif

#include <cmath>
#include <vector>

#define forn(i, n) for (int i = 0; i < (int)(n); i++)
#define pb push_back

const double eps = 1.1e-12;

const int N = 5e4;                  
int n, x[N], y[N];
std::vector<int> c[N];
int cnt, cc, u[N];

inline void dfs( int v )
{
  if (u[v] == cc)
    return;
  u[v] = cc, cnt++;
  forn(i, c[v].size())
    dfs(c[v][i]);  
}

void readInput( InStream &in )
{
  n = in.readInt(1, N);
  forn(i, n)
  {
    x[i] = in.readInt();
    y[i] = in.readInt();
  }
}

double sqr( double x )
{
  return x * x;
}

double dist( int a, int b )
{
  return sqrt(sqr(x[a] - x[b]) + sqr(y[a] - y[b]));
}

double checkOutput( InStream &out )
{
  double res = out.readDouble(), sum = 0, err;
  forn(i, n)
    c[i].clear();
  forn(i, n - 1)
  {
    int a = out.readInt(1, n) - 1;
    int b = out.readInt(1, n) - 1;
    c[a].pb(b), c[b].pb(a);
    sum += dist(a, b);
  }
  cnt = 0, cc++;
  dfs(0);
  if (cnt != n)
    out.quitf(_wa, "output is not a tree");
  ensure(sum != 0);
  if ((err = fabs(res / sum - 1)) > eps)
    out.quitf(_wa, "contestant says summary weight is %.13f, but in real it's %.13f, error = %g", res, sum, err);
  return res;
}

int main(int argc, char *argv[])
{
  registerTestlibCmd(argc, argv);

  readInput(inf);

  double ja = checkOutput(ans);
  double ca = checkOutput(ouf);
  ensure(ja != 0);
  double err = fabs(ca / ja - 1);
  if (err > eps)
    quitf(ca < ja ? _fail : _wa, "contestant has weight %.13f, jury has weight %.13f, error = %g", ca, ja, err);
  quitf(_ok, "OK (n = %d, answer = %.13f, error = %g)", n, ja, err);
  return 0;
}
