module randommst_solution;
version = IO_FILES;
import std.algorithm;
import std.container;
import std.conv;
import std.math;
import std.range;
import std.stdio;
import std.typecons;

immutable string PROBLEM_NAME = "randommst";
immutable int NA = -1;
immutable int AROUND = 400;

alias pair = Tuple !(int, "x", int, "y");
alias point = Tuple !(int, "x", int, "y", int, "k");

int rho2 (const ref point a, const ref point b) pure nothrow
{
	return (a.x - b.x) ^^ 2 + (a.y - b.y) ^^ 2;
}

void main ()
{
	version (IO_FILES)
	{
		stdin  = File (PROBLEM_NAME ~ ".in",  "rt");
		stdout = File (PROBLEM_NAME ~ ".out", "wt");
	}

	int n;
	while (readf (" %s", &n) > 0)
	{
		auto p = new point [n];
		foreach (i, ref z; p)
		{
			readf (" %s %s", &z.x, &z.y);
			z.k = i;
		}
		sort (p);

		auto d = new int [n];
		d[] = int.max;
		d[0] = 0;
		auto r = new int [n];
		r[] = NA;
		auto b = new bool [n];
		b[] = false;
		auto s = redBlackTree (new pair [0]);
		foreach (i; 0..n)
		{
			s.insert (pair (d[i], i));
		}

		real res = 0;
		pair [] ans;
		foreach (t; 0..n)
		{
			auto cur = s.front;
			int pos = cur.y;
			s.removeFront ();
			b[pos] = true;
			if (r[pos] != NA)
			{
				ans ~= pair (p[r[pos]].k, p[pos].k);
				res += sqrt (to !(real) (cur.x));
			}
			int lo = max (0, pos - AROUND);
			int hi = min (n - 1, pos + AROUND);
			foreach (i; lo..hi + 1)
			{
				if (b[i])
				{
					continue;
				}
				int e = rho2 (p[pos], p[i]);
				debug {writeln (pos, ' ', i, ' ', e);}
				if (d[i] > e)
				{
					s.removeKey (pair (d[i], i));
					d[i] = e;
					r[i] = pos;
					s.insert (pair (d[i], i));
				}
			}
		}

		writefln ("%.20f", res);
		foreach (e; ans)
		{
			writefln ("%s %s", e.x + 1, e.y + 1);
		}
	}
}
