/**
 * Author: Sergey Kopeliovich (Burunduk30@gmail.com)
 * Date: 2014.04.06
 * OK, Time = 0.62
 */

#include <cstdio>
#include <cmath>
#include <cassert>
#include <algorithm>

using namespace std;

#define forn(i, n) for (int i = 0; i < (int)(n); i++)

typedef pair <int, int> pii;

const int N = 5e4;
const int K = 13;

struct edge
{
  int a, b, w;
  bool operator < ( const edge &a ) const { return w < a.w; }
} e[N * K];

double sum = 0, z[N];
int cnt, dn, en, k, n, x[N], y[N], p[N];
pii d[K + 1];

template <class T>
inline T sqr( T x ) { return x * x; }
inline int D( int i, int j ) { return sqr(x[i] - x[j]) + sqr(y[i] - y[j]); }

template <int maxN> struct DSU
{
  int p[maxN];
  int get( int a )          { return a == p[a] ? a : (p[a] = get(p[a])); }
  void join( int a, int b ) { p[get(a)] = get(b); }
  void init( int n )        { forn(i, n) p[i] = i; }
};

DSU<N> dsu;

int main()
{
  #define NAME "randommst"
  assert(freopen(NAME ".in", "r", stdin));
  assert(freopen(NAME ".out", "w", stdout));

  double ang = 0.123, co = cos(ang), si = sin(ang);
  assert(scanf("%d", &n) == 1 && n <= N);
  forn(i, n)
  {
    scanf("%d%d", &x[i], &y[i]);
    z[i] = x[i] * co + y[i] * si;
    p[i] = i;
  }
  sort(p, p + n, []( int i, int j ){ return z[i] < z[j]; });

  forn(i, n)
  {
    dn = 0;
    for (int j = i + 1; j < n; j++)
    {
      cnt++;
      int tmp = D(p[i], p[j]), pos = dn;
      while (pos && d[pos - 1].first > tmp)
        d[pos] = d[pos - 1], pos--;
      d[pos] = pii(tmp, p[j]);
      dn += (dn < K);
      if (sqr(z[p[i]] - z[p[j]]) >= d[dn - 1].first)
        break;
    }
    forn(j, dn)
      e[en++] = {p[i], d[j].second, d[j].first};
  }
  sort(e, e + en);
  fprintf(stderr, "n = %d, cnt = %d, en = %d\n", n, cnt, en);

  dsu.init(n);
  forn(i, en)
    if (dsu.get(e[i].a) != dsu.get(e[i].b))
    {
      dsu.join(e[i].a, e[i].b);
      sum += sqrt(e[i].w);
      e[k++] = e[i];
    }
  assert(k == n - 1);

  printf("%.20f\n", sum);
  forn(i, n - 1)
    printf("%d %d\n", e[i].a + 1, e[i].b + 1);
  return 0;
}
