/**
 * Author: Sergey Kopeliovich (Burunduk30@gmail.com)
 * Date: 2014.04.13
 * OK, Time = 3.25
 */

import java.io.*;
import java.util.*;

class randommst_sk_good {

  static long time() {
    return System.currentTimeMillis();
  }

  final int N = (int)5e4;
  final int K = 13;

  class Edge implements Comparable<Edge> {
    int a, b, w;
    public int compareTo( Edge e ) {
      return w - e.w;
    }
    public Edge( int a, int b, int w ) {
      this.a = a;
      this.b = b;
      this.w = w;
    }
  }
  
  Edge e[] = new Edge[N * K];
  Edge d[] = new Edge[K + 1];

  double sum = 0;
  int cnt, dn, en, k, n, x[], y[];

  int sqr( int x ) { return x * x; }
  double sqrd( double x ) { return x * x; }
  int D( int i, int j ) { return sqr(x[i] - x[j]) + sqr(y[i] - y[j]); }

  class DSU {
    int p[];
    public int get( int a )      { return a == p[a] ? a : (p[a] = get(p[a])); }
    public void join( int a, int b ) { p[get(a)] = get(b); }
    public void init( int n )        { 
      p = new int[n];
      for (int i = 0; i < n; i++)
        p[i] = i; 
    }
  }

  DSU dsu = new DSU();

  class Box implements Comparable<Box> {
    int ind;
    double len;
    public Box( int ind, double len ) {
      this.ind = ind;
      this.len = len;
    }
    public int compareTo( Box b ) {
      if (len < b.len) return -1;
      if (len > b.len) return +1;
      return 0;
    }
  }

  public static void main( String[] args ) throws Exception {
    new randommst_sk_good().run();
  }
   
  public void run() throws Exception {
    Locale.setDefault(Locale.US);

    final String NAME = "randommst";
    MyReader in = new MyReader(new FileInputStream(NAME + ".in"));
    PrintWriter out = new PrintWriter(new File(NAME + ".out"));

    long start = time();

    double ang = 0.123, co = Math.cos(ang), si = Math.sin(ang);
    n = in.nextInt();
    x = new int[n];
    y = new int[n];
    Box z[] = new Box[n];
    for (int i = 0; i < n; i++) {
      x[i] = in.nextInt();
      y[i] = in.nextInt();
      z[i] = new Box(i, x[i] * co + y[i] * si);
    }
    Arrays.sort(z);

    System.err.format("time = %.2f : data is read\n", (time() - start) * 1e-3);

    for (int i = 0; i < n; i++) {
      dn = 0;
      for (int j = i + 1; j < n; j++) {
        cnt++;
        int tmp = D(z[i].ind, z[j].ind), pos = dn;
        while (pos > 0 && d[pos - 1].w > tmp) {
          d[pos] = d[pos - 1];
          pos--;
        }
        d[pos] = new Edge(z[i].ind, z[j].ind, tmp);
        if (dn < K)
          dn++;
        if (sqrd(z[i].len - z[j].len) >= d[dn - 1].w)
          break;
      }
      for (int j = 0; j < dn; j++)
        e[en++] = d[j];
    }
    System.err.format("time = %.2f : edges are calced [n = %d, cnt = %d, en = %d]\n", (time() - start) * 1e-3, n, cnt, en);
    Arrays.sort(e, 0, en);
    System.err.format("time = %.2f : edges are sorted\n", (time() - start) * 1e-3);

    dsu.init(n);
    for (int i = 0; i < en; i++)
      if (dsu.get(e[i].a) != dsu.get(e[i].b)) {
        dsu.join(e[i].a, e[i].b);
        sum += Math.sqrt(e[i].w);
        e[k++] = e[i];
      }

    System.err.format("time = %.2f : MST is built\n", (time() - start) * 1e-3);

    out.format("%.20f\n", sum);
    for (int i = 0; i < n - 1; i++)
      out.format("%d %d\n", e[i].a + 1, e[i].b + 1);
    out.close();

    System.err.format("time = %.2f : result is printed\n", (time() - start) * 1e-3);
  }

  class MyReader {
    BufferedInputStream in;

    final int bufSize = 1 << 16;
    final byte b[] = new byte[bufSize];

    MyReader( InputStream in ) {
      this.in = new BufferedInputStream(in, bufSize);
    }

    int nextInt() throws IOException {
      int c;
      while ((c = nextChar()) < '0')
        ;
      int x = c - '0';
      while ((c = nextChar()) >= '0')
        x = x * 10 + (c - '0');
      return x;
    }

    int bn = bufSize, k = bufSize;
    
    int nextChar() throws IOException {
      if (bn == k) {
        in.read(b, 0, bufSize);
        bn = 0;
      }
      return b[bn++];
    }
  }
}
